@@ -13,20 +13,35 @@
 proc Exwin_Init {} {
     global exwin
     Preferences_Resource exwin(mtextLines) mtextLines 25
-    Preferences_Add "Window Stuff" \
-"Unfortunatly you cannot directly change the size of interior windows, so you have to pick the size of the folder display via this interface.  To change the size of the message display, just stretch the main window and exmh will remember that size.  The scrollbar side can be \"left\" or \"right\"." {
-	{exwin(scrollbarSide) scrollbarSide right {Scrollbar side}
+    Preferences_Add "Windows & Scrolling" \
+"Window placement and scroll-related preferences are set here.
+Contrained scrolling keeps the last line of text in a window stuck
+to the bottom of the window.  This can be set independently for
+the scan listing (FTOC)." {
+	{exwin(scrollbarSide) scrollbarSide {CHOICE right left} {Scrollbar side}
 "Which side the scrollbars appear on.  This
-only takes effect after you restart exmh.
-Set to \"left\" or \"right\"."}
+only takes effect after you restart exmh."}
 	{exwin(scrollSpeed) scrollSpeed 1 {Drag-Scroll speed}
 "How fast things scroll when you drag a text widget
 around with the (by default) middle button."}
 	{exwin(scrollAccel) scrollAccel 4 {Drag-Scroll acceleration}
 "How fast things scroll when you drag a text widget
 around with the (by default) middle button with Shift depressed."}
-	{exwin(ftextLines)	ftextLines 15	{Folder lines}
-"Lines in the folder display window."}
+    {widgetText(constrained) textConstrainScroll OFF "Constrained Scrolling"
+"Constrained scrolling clamps the last line
+of text to the bottom of the text widget."} 
+    {widgetText(constrainFtoc) textConstrainFtoc ON "Constrain FTOC"
+"In spite of the general constrained text setting,
+constrain the scrolling of the folder table-of-contents."}
+    {widgetText(scrollContext) textContextLines 2 "Scroll Context"
+"Scroll context is the number of lines that pages
+overlap when paging up and down through text."}
+    {widgetText(selectDelay) textSelectDelay 50 "Scroll/Select Time Constant"
+"When you drag a selection off the top or bottom of a text widget,
+the widget automatically scrolls and extends the selection.  This
+parameter is a polling time period that affects the behavior.  Too
+big and it is not responsive enough.  Too small and it may consume too
+many cycles.  Units are milliseconds."}
 	{exwin(placeToplevel)	placeToplevel ON	{Remember window placement}
 "With this enabled, exmh will remember the placement of the various
 popup windows between sessions.  This means you can position them
@@ -36,7 +51,7 @@ then a remembered placement might be in the wrong room.  You can nuke
 all the placement memory from the end of the .exmh-defaults file and
 always run exmh from the same room, or just disable this feature.."}
     }
-
+    set exwin(ftextLinesSave) $exwin(ftextLines)
     trace variable exwin(ftextLines) w ExwinFixupFtextLines
 #    trace variable exwin(mtextLines) w ExwinFixupMtextLines
     if ![info exists exwin(toplevels)] {
@@ -46,7 +61,14 @@ always run exmh from the same room, or just disable this feature.."}
 proc ExwinFixupFtextLines { args } {
     global exwin
     Exmh_Debug ExwinFixupFtextLines $exwin(ftextLines)
-    $exwin(ftext) configure -height $exwin(ftextLines)
+    if [catch {
+	$exwin(ftext) configure -height $exwin(ftextLines)
+	set exwin(ftextLinesSave) $exwin(ftextLines)
+    } err] {
+	Exmh_Status $err
+	set exwin(ftextLines) $exwin(ftextLinesSave)
+	$exwin(ftext) configure -height $exwin(ftextLines)
+    }
 }
 #proc ExwinFixupMtextLines { args } {
 #    global exwin
@@ -59,6 +81,7 @@ proc Exwin_Layout {} {
 
     set fixed {top fill}
     set expand {top fill expand}
+    set exwin(grab) {}
 
     wm minsize . 100 100
     Flag_Init
@@ -86,16 +109,22 @@ proc Exwin_Layout {} {
     set mid     [Widget_Frame . mid Mid $fixed]
     Widget_SplitFrameR $mid Face Right
     set face $mid.left
-    canvas $face.c -background white -width 48 -height 48
+    canvas $face.c -width 48 -height 48
     set exwin(faceCanvas) $face.c
     # default image
-    $face.c create bitmap 0 0 -anchor nw -bitmap @$exmh(library)/exmh.bitmap
+    set exwin(faceFg) [option get $face.c foreground Foreground]
+    $face.c create bitmap 0 0 -anchor nw -bitmap @$exmh(library)/exmh.bitmap \
+	-foreground $exwin(faceFg)
     pack append $face $face.c {top}
 
     # Status line + MsgID
     set right $mid.right
     Widget_SplitFrameV $right Status Mops
     set exwin(status) [Widget_Entry $right.top msg {right expand fill}]
+    set statusConfig [option get . statusConfig StatusConfig]
+    if {[string length $statusConfig] > 0} {
+        catch [concat $exwin(status) configure $statusConfig]
+    }
     Label_MessageSetup $right.top
     # Status line does double-duty for folder/msg selection typein
     Select_EntryBind $exwin(status)
@@ -109,7 +138,15 @@ proc Exwin_Layout {} {
 				$exwin(mtextLines)]
     Msg_Setup $exwin(mtext)
     Bindings_Main $exwin(mtext)
-    focus $exwin(mtext)
+
+    #
+    # Focus handling.  The message display is the default focus
+    # because all the command keys are bound there.
+    # The <FocusIn> binding is for when the user has "focus-follows-mouse"
+    # and moves the mouse into the main window from, say, the sedit window.
+
+    focus default $exwin(mtext)
+    bind . <FocusIn> [list focus $exwin(mtext)]
 }
 proc Exwin_FullFtoc {} {
     global exwin
@@ -141,6 +178,9 @@ proc Exwin_FullFtoc {} {
 }
 
 proc Exwin_IconPosition { w icon } {
+    if {[string length $icon] == 0} {
+	return	;# Don't mess
+    }
     # icon looks like +x+y, or -x-y, etc.
     set x 0 ; set y 0
     if {[llength $icon] == 1} {
@@ -214,6 +254,41 @@ proc Exwin_Dismiss { path {geo ok} } {
     wm withdraw $path
     Exmh_Focus    
 }
+proc Exwin_ToplevelFocus { toplevel {default none} } {
+    global exwin
+    if {[string compare $default none] != 0} {
+	focus $default
+    }
+    set exwin(focus,$toplevel) $default
+    bind $toplevel <FocusIn> {ExwinSetFocus %W}
+    bind $toplevel <FocusOut> {ExwinSaveFocus %W}
+}
+proc ExwinSetFocus { w } {
+    global exwin
+    catch {
+	focus $exwin(focus,$w) ;# could be "none", or empty
+    }
+}
+proc ExwinSaveFocus { w } {
+    global exwin
+    set new [focus]
+    if {[string compare $new none] == 0} {
+	return
+    }
+    if {[string compare [winfo toplevel $new] $w] == 0} {
+	set exwin(focus,$w) $new
+    }
+}
+proc Exwin_Grab { args } {
+    global exwin
+    Exmh_Debug Exwin_Grab $args focus=[focus]
+    if {[string compare [lindex $args 0] release] == 0} {
+	set exwin(grab) {}
+    } else {
+	set exwin(grab) $args
+    }
+    eval grab $args
+}
 proc Exwin_CheckPoint { } {
     global exwin
     if {! $exwin(placeToplevel)} {
@@ -221,7 +296,7 @@ proc Exwin_CheckPoint { } {
 	return
     }
     set oldstuff [Preferences_ReadSection "Saved Window Positions" "End Positions"]
-    set newstuff [list [format "*exwinPaths:\t%s" $exwin(toplevels)]]
+    set newstuff {}
     foreach path $exwin(toplevels) {
 	set npath [string trimleft $path .]
 	if [catch {wm state $path} state] {
@@ -248,9 +323,15 @@ proc Exwin_CheckPoint { } {
 		}
 	    }
 	}
-	lappend newstuff [format "*%s.position:\t%s" $npath \
+	if {[string length $geo] != 0} {
+	    lappend newstuff [format "*%s.position:\t%s" $npath \
 			[string trimleft $geo -x0123456789]]
+	} else {
+	    set ix [lsearch $exwin(toplevels) $path]
+	    set exwin(toplevels) [lreplace $exwin(toplevels) $ix $ix]
+	}
     }
+    lappend newstuff [format "*exwinPaths:\t%s" $exwin(toplevels)]
     lappend newstuff [format "%s.geometry:\t%s" [winfo name .] [wm geometry .]]
     Fdisp_Checkpoint newstuff
     Preferences_RewriteSection "Saved Window Positions" "End Positions" $newstuff
