@@ -30,10 +30,9 @@ proc Flist_Init {} {
 proc FlistInitVars {} {
     global flist
     set flist(unseen) {}	;# Sequence of folders to visit
+    set flist(unvisited) {}	;# Unseen folders not yet visited
     set flist(newMsgs) 0
     set flist(newFolders) 0
-    #   flist(new,$folder)	;# not always defined
-    #   flist(newseq,$folder)	;# not always defined
 }
 
 ### Routines to find all folders, figure out which have nested folders, etc.
@@ -66,7 +65,7 @@ proc Flist_FindAllFolders {} {
 proc FlistFindAllInner {} {
     global flist flistSubCache flistParents mhProfile
     Widget_Toplevel .scanning "Scanning..."
-    Widget_Message .scanning msg  -text "
+    Widget_Message .scanning msg -cursor watch -text "
 Scanning for nested folders.
 (folders -fast -recurse)
 This can be very slow if you have
@@ -81,9 +80,27 @@ button to update the folder set.
 
 Please wait...
 "
-    Exmh_Status "Scanning for nested folders ..." purple
+    Exmh_Status "Scanning for nested folders ..." warn
     update
-    set flist(allfolders) [FlistSort [split [exec folders -fast -recurse] \n]]
+    set bogus [catch {exec folders -fast -recurse} raw]
+    set raw [split $raw \n]
+    if {$bogus} {
+	set ix [lsearch -glob $raw "* * *"]
+	if {$ix >= 0} {
+	    set msg [lindex $raw $ix]
+	    .scanning.msg config -text $msg
+	    Exmh_Status $msg
+	    catch {puts stderr $raw}
+	    update
+	    after 1000
+	    set raw [lreplace $raw $ix $ix]
+	} else {
+	    Exmh_Status "Folders error report on stderr"
+	    catch {puts stderr $raw}
+	}
+    }
+
+    set flist(allfolders) [FlistSort $raw]
     FlistSubFoldersInit
     FlistCacheFolderList
     destroy .scanning
@@ -225,8 +242,6 @@ proc Flist_UnseenFolders {} {
 proc FlistReset {} {
     global flist
     FlistInitVars
-    
-    Exmh_Debug FlistReset
 }
 proc Flist_AddUnseen {folder seq} {
     global flist exmh
@@ -234,7 +249,6 @@ proc Flist_AddUnseen {folder seq} {
     if {$num <= 0} {
 	return
     }
-    Exmh_Debug Flist_AddUnseen $folder $seq
     if {$folder == $exmh(folder)} {
 	Flist_UpdateUnseen $folder $seq
 	return
@@ -244,11 +258,11 @@ proc Flist_AddUnseen {folder seq} {
     set flist(newseq,$folder) $seq
     incr flist(newFolders)
     lappend flist(unseen) $folder
+    lappend flist(unvisited) $folder
     Fdisp_HighlightUnseen $folder
 }
 proc Flist_UpdateUnseen {folder seq} {
     global flist exmh
-    Exmh_Debug Flist_UpdateUnseen $folder $seq seen=[Msg_Seen]
     # Check overlap with already seen msgs
     foreach id [Msg_Seen] {
 	set ix [lsearch $seq $id]
@@ -267,6 +281,9 @@ proc Flist_UpdateUnseen {folder seq} {
 	incr flist(newFolders)
 	lappend flist(unseen) $folder
     }
+    if {[lsearch $flist(unvisited) $folder] < 0} {
+	lappend flist(unvisited) $folder
+    }
     Fdisp_HighlightUnseen $folder
 }
 proc Flist_Done {} {
@@ -279,7 +296,6 @@ proc Flist_Done {} {
 	    set flist(lastNewMsgs) 0
 	}
 	set delta [expr {$flist(newMsgs) - $flist(lastNewMsgs)}]
-	Exmh_Debug "Flist_Done deltaNew $delta"
 	if {$delta > 0} {
 	    Flag_NewMail
 	    Sound_Feedback $delta
@@ -294,7 +310,6 @@ proc Flist_Done {} {
 
 # Flist enumerates folders that have unseen messages.
 proc Flist_FindUnseen {} {
-    Exmh_Debug Flist_FindUnseen begin
     Exmh_Debug Flist_FindUnseen end [time FlistFindUnseen]
 }
 
@@ -303,6 +318,16 @@ proc FlistFindUnseen {} {
     BgRPC FlistReset
     FlistGetContext
     foreach f $flist(allfolders) {
+	set check 0
+	foreach pat $mhProfile(folder-unseen) {
+	    if [string match $pat $f] {
+		set check 1
+		break
+	    }
+	}
+	if {! $check} {
+	    continue
+	}
 	set hit 0
 	foreach line $flist(context) {
 	    set line [split $line]
@@ -329,12 +354,11 @@ proc FlistFindUnseen {} {
 }
 proc FlistGetContext {} {
     global flist mhProfile
-    if {$flist(contextMtime) < [file mtime $mhProfile(path)/$mhProfile(context)]} {
-	if {[catch {open $mhProfile(path)/$mhProfile(context)} in] == 0} {
+    if {$flist(contextMtime) < [file mtime $mhProfile(context)]} {
+	if {[catch {open $mhProfile(context)} in] == 0} {
 	    set flist(context) [split [read $in] \n]
-	    set flist(contextMtime) [file mtime $mhProfile(path)/$mhProfile(context)]
+	    set flist(contextMtime) [file mtime $mhProfile(context)]
 	    close $in
-	    Exmh_Debug "Reloaded $mhProfile(path)/$mhProfile(context) at $flist(contextMtime)"
 	}
     }
 }
@@ -358,6 +382,7 @@ proc FlistUnseenFolder { folder } {
     global flist
     unset flist(new,$folder)
     unset flist(newseq,$folder)
+    Exmh_Debug FlistUnseenFolder $folder
     Fdisp_UnHighlightUnseen $folder
     set ix [lsearch $flist(unseen) $folder]
     if {$ix >= 0} {
@@ -367,6 +392,10 @@ proc FlistUnseenFolder { folder } {
 	    Flag_NoUnseen
 	}
     }
+    set ix [lsearch $flist(unvisited) $folder]
+    if {$ix >= 0} {
+	set flist(unvisited) [lreplace $flist(unvisited) $ix $ix]
+    }
 }
 proc Flist_Update { folder {delete no} } {
     global flist
@@ -412,6 +441,10 @@ proc FlistUpdate { folder {delete no} } {
 	if {$ix >= 0} {
 	    set flist(unseen) [lreplace $flist(unseen) $ix $ix]
 	}
+	set ix [lsearch $flist(unvisited) $folder]
+	if {$ix >= 0} {
+	    set flist(unvisited) [lreplace $flist(unvisited) $ix $ix]
+	}
     }
 }
 
@@ -434,7 +467,6 @@ proc FlistSort { dirlist } {
 	    set patLen [string length $pat]
 	    if {$patLen > $patLength($f)} {
 		if [string match $pat $f] {
-#    puts stderr "$f matchs $pat at pri $pri"
 		    set priority($f) $pri
 		    set patLength($f) $patLen
 		    set hit 1
@@ -468,3 +500,32 @@ proc FlistSort { dirlist } {
     }
     return $result
 }
+
+proc Flist_NextUnseen { } {
+    # Return the next folder in Folder-Order that has unseen messages
+    global flist exmh
+
+    foreach f $flist(unvisited) {
+	if {[string compare $f $exmh(folder)] != 0} {
+	    return $f
+	}
+    }
+    foreach f $flist(unseen) {
+	if {[string compare $f $exmh(folder)] != 0} {
+	    return $f
+	}
+    }
+    set first [lindex $flist(allfolders) 0]
+    if {$flist(cycleBack) && [string compare $first $exmh(folder)]} {
+	return $first
+    } else {
+	return {}
+    }
+}
+proc Flist_Visited { f } {
+    global flist
+    set ix [lsearch $flist(unvisited) $f]
+    if {$ix >= 0} {
+	set flist(unvisited) [lreplace $flist(unvisited) $ix $ix]
+    }
+}
