@@ -46,6 +46,9 @@ after being iconic."}
 "This option causes a message to be generated each time a
 new message is sorted into a folder.  There isn't much info,
 it's just sort of a heart-beat feature..."}
+	{inc(xnsgetmail) xnsGetMail OFF	  {Run xnsgetmail -k}
+"Run xnsgetmail -k before inc in order to fetch your
+XNS mail messages into your UNIX spool file."}
     }
 }
 proc Inc_Startup {} {
@@ -68,6 +71,9 @@ proc Inc_Mapped {} {
 proc Inc {} {
     # There are three different styles of inc'ing messages...
     global inc
+    if $inc(xnsgetmail) {
+	Xns_GetMail
+    }
     case $inc(style) {
 	{default inbox}	{ busy Inc_Inbox }
 	presort		{ busy Inc_Presort }
@@ -78,13 +84,13 @@ proc Inc {} {
 
 proc Inc_Inbox {} {
     # Inc from the spool file to the inbox folder
-    global exmh mhProfile
+    global exmh mhProfile ftoc
     if [info exists mhProfile(inbox)] {
 	set inbox [string trimleft $mhProfile(inbox) +]
     } else {
 	set inbox inbox
     }
-    if [catch {exec inc +$inbox -truncate -nochangecur} incout] {
+    if [catch {exec inc +$inbox -truncate -nochangecur -width $ftoc(scanWidth)} incout] {
 	Exmh_Debug Inc_Inbox +${inbox}: $incout
 	set incout {}
     }
@@ -118,35 +124,39 @@ proc Inc_Presort {} {
     # Transfer from the mail spool file directly into folders
     global exmh mhProfile env inc
     # Just use inc to copy stuff into a temp directory
-    exec folder -push +[exec folder -fast] < /dev/null > /dev/null
+    catch {exec folder -push +[exec folder -fast] < /dev/null > /dev/null}
     exec folder +MyIncTmp < /dev/null > /dev/null
     if {[catch {exec inc +MyIncTmp -silent} err]} {
 	# Some versions of inc exit non-zero when they should not.
 	Exmh_Debug Inc_Presort +MyIncTmp: $err
     }
+    if [catch {set env(USER)} user] {
+	if [catch {set env(LOGNAME)} user] {
+	    Exmh_Status "No USER or LOGNAME envar"
+	    set user ""
+	}
+    }
     if {[catch {glob $mhProfile(path)/MyIncTmp/*} files] == 0} {
 	Exmh_Status "presort inc ..." red
-	set rmFiles {}
 	foreach file $files {
 	    if {$file == "++"} {
-		lappend rmFiles $file
+		if [catch {exec rm $file} err] {
+		    Exmh_Debug Inc_Presort: $err
+		}
 		continue
 	    }
 	    # now use slocal to (presumably) shift things to a folder
-	    if [catch {exec $exmh(slocal) -user $env(USER) < $file}] {
+	    if [catch {exec $exmh(slocal) -user $user < $file}] {
 		Exmh_Status "Cannot classify $file"
 	    } else {
-		lappend rmFiles $file
+		if [catch {exec rm $file} err] {
+		    Exmh_Debug Inc_Presort: $err
+		}
 		if {$inc(presortFeedback)} {
 		    Exmh_Status [file tail $file] red
 		}
 	    }
 	}
-	if [llength $rmFiles] {
-	    if [catch {eval exec rm $rmFiles} err] {
-		Exmh_Debug Inc_Presort: $err
-	    }
-	}
     }
     if [catch {exec folder -pop < /dev/null > /dev/null} err] {
 	Exmh_Debug $err
@@ -158,11 +168,10 @@ proc Inc_Presort {} {
     BgRPC Inc_PresortFinish
 }
 proc Inc_PresortFinish {} {
-    global exmh
+    global exmh ftoc
     Mh_Folder $exmh(folder)	;# prestort inc has changed this to MyIncTmp
     if {[Flist_NumUnseen $exmh(folder)] > 0} {
-	Scan_Folder $exmh(folder)
-	Msg_ShowCurrent
+	Scan_Folder $exmh(folder) $ftoc(showNew)
     }
 }
 
@@ -171,26 +180,27 @@ proc Inc_PresortFinish {} {
 # This also works with POP hosts.
 
 proc Inc_All {} {
-    global exdrops exmh
+    global exdrops exmh ftoc
 
     Exmh_Status "Checking dropboxes..." red
+    MhSetMailDrops	;# Update, if needed
     set hits {}
     foreach folder [array names exdrops] {
 	set dropname $exdrops($folder)
 	if {[string first / $dropname] < 0} {
 	    # Not a pathname, but a POP hostname
 	    set host [lindex $dropname 0]
-	    Exmh_Status $folder @ $host
+	    Exmh_Status "$folder @ $host"
 	    if {[llength $dropname] == 2} {
 		set user [lindex $dropname 1]
-		set cmd [list exec inc +$folder -host $host -user $user -silent -truncate]
+		set cmd [list exec inc +$folder -host $host -user $user -truncate -width $ftoc(scanWidth)]
 	    } else {
-		set cmd [list exec inc +$folder -host $host -truncate]
+		set cmd [list exec inc +$folder -host $host -truncate -width $ftoc(scanWidth)]
 	    }
 	} else {
 	    if { [file exists $dropname] && [file size $dropname] } {
 		lappend hits $folder
-		set cmd [list exec inc +$folder -file $dropname -truncate ]
+		set cmd [list exec inc +$folder -file $dropname -truncate -width $ftoc(scanWidth)]
 	    } else {
 		set cmd {}
 	    }
