@@ -22,8 +22,7 @@ proc Pick {} {
 	Widget_AddBut $f pick "Pick" {Pick_It} {left padx 1}
 	set pick(addtosel) 0
 	Widget_CheckBut $f add "Add to Sel" pick(addtosel) {left padx 1}
-	set pick(project) 0
-	Widget_CheckBut $f project "New FTOC" pick(project) {left padx 1}
+	Widget_AddBut $f project "New FTOC" Ftoc_NewFtoc {left padx 1}
     
 	set pick(panes) 0
 	PickButtons
@@ -33,20 +32,32 @@ proc PickButtons {} {
     global pick
 
     incr pick(panes)
+    if {$pick(panes) == 1} {
+	catch {unset pick(lastentry)}
+    }
     set f [Widget_Frame .pick rim$pick(panes) Rim]
 
     $f configure -bd 10
 
-    PickEntry $f subject Subject
-    PickEntry $f from From
-    PickEntry $f to To
-    PickEntry $f cc Cc
-    PickEntry $f before Before
-    PickEntry $f after After
-    PickEntry $f search Search
+    foreach var {subject from to cc before after search msgs sequence} {
+	set pick($pick(panes),$var) {}
+    }
+
+    Widget_BeginEntries 15 25 Pick_It
+    Widget_LabeledEntry $f.subject	Subject pick($pick(panes),subject)
+    Widget_LabeledEntry $f.from		From pick($pick(panes),from)
+    Widget_LabeledEntry $f.to		To pick($pick(panes),to)
+    Widget_LabeledEntry $f.cc		Cc pick($pick(panes),cc)
+    Widget_LabeledEntry $f.before	Before pick($pick(panes),before)
+    Widget_LabeledEntry $f.after	After pick($pick(panes),after)
+    Widget_LabeledEntry $f.search	Search pick($pick(panes),search)
+    Widget_LabeledEntry $f.msgs		"Sequence/msgs" pick($pick(panes),msgs)
     set pad [Widget_Frame $f pad Pad]
     $pad configure -height 10 -width 10
-    PickEntry $f msgs Messages
+    Widget_LabeledEntry $f.sequence	"Add to Sequence" pick($pick(panes),sequence)
+    Widget_EndEntries	;# This sets focus
+
+    Exwin_ToplevelFocus .pick [focus]
 
     Widget_AddBut $f or "Or" { PickOr }
 }
@@ -64,15 +75,6 @@ proc PickOr {} {
 
     PickButtons
 }
-proc PickEntry { frame name label } {
-    global pick
-    set f1 [Widget_Frame $frame $name $label]
-    $f1 configure -bd 2 -relief raised
-    Widget_Label $f1 label {left fill} -text ${label}:
-    set pick(entry,$pick(panes),$name) [Widget_Entry $f1 entry {left expand fill}]
-    bind $pick(entry,$pick(panes),$name) <Return> {Pick_It}
-}
-
 proc PickClear {} {
     global pick
     for {set pane 1} {$pane <= $pick(panes)} {incr pane} {
@@ -90,7 +92,7 @@ proc Pick_It {} {
     for {set pane 1} {$pane <= $pick(panes)} {incr pane} {
 	set and 0
 	foreach field {subject from to cc before after search} {
-	    set text [$pick(entry,$pane,$field) get]
+	    set text $pick($pane,$field)
 	    if {$text != {}} {
 		if {$inpane != $pane} {
 		    if {$hadpane} {
@@ -107,7 +109,7 @@ proc Pick_It {} {
 	        set and 1
 	    }
 	}
-	set m [$pick(entry,$pane,msgs) get]
+	set m $pick($pane,msgs)
 	if {$m != {}} {
 	    lappend msgs $m
 	}
@@ -115,6 +117,12 @@ proc Pick_It {} {
 	    lappend cmd -rbrace
 	}
     }
+    incr pane -1
+    set seq $pick($pane,sequence)
+    if {$seq != {}} {
+	lappend msgs -sequence $seq
+    }
+
     Exmh_Debug Pick_It $cmd $msgs
     busy PickInner $cmd $msgs
     Exmh_Focus
@@ -131,14 +139,8 @@ proc PickInner {cmd msgs} {
     if {! $pick(addtosel)} {
 	Ftoc_RangeUnHighlight
     }
-    if {$pick(project)} {
-	if {[Ftoc_Changes pick] == 0} {
-	    Scan_ProjectSelection $pick(ids)
-	}
-    } else {
-	Ftoc_PickMsgs $pick(ids)
-    }
-    Exmh_Status "Pick hit [llength $ids] msgs" blue
+    Ftoc_PickMsgs $pick(ids) $pick(addtosel)
+    Exmh_Status "Pick hit [llength $pick(ids)] msgs" blue
 }
 proc PickMarkSeen {} {
     global exmh pick
